$column-horizontal-spacing: 10px;

.logs-panel-options {
  display: flex;
  background-color: $page-bg;
  padding: $space-sm $space-md $space-sm $space-md;
  border-radius: $border-radius;
  margin: $space-md 0 $space-sm;
  border: $panel-border;
  flex-direction: column;
}

.logs-panel-controls {
  display: flex;
  justify-content: space-between;
  align-items: baseline;
  flex-wrap: wrap;
  .logs-panel-controls-main {
    display: flex;
    justify-items: flex-start;
    align-items: center;
    flex-wrap: wrap;

    > * {
      margin-right: $spacer * 2;
    }
  }
}

.logs-panel-nodata {
  > * {
    margin-left: 0.5em;
  }
}
