.panel-wrapper {
  height: 100%;
  position: relative;

  &--view {
    flex: 1 1 0;
    height: 90%;
  }
}

.panel-editor__scroll {
  flex-grow: 1;
  min-width: 0;
  display: flex;
  min-height: 0;
  height: 100%;
  overflow: hidden;
}

.panel-editor__content {
  padding: 0 16px 16px 16px;
}

.panel-in-fullscreen {
  .sidemenu {
    display: none;
  }

  .search-container {
    left: 0 !important;
  }
}

.ds-picker {
  position: relative;
  min-width: 200px;
  max-width: 300px;
}
